#ifndef _TARGETVER_H
#define _TARGETVER_H

#include <winsdkver.h>

#define _WIN32_WINNT _WIN32_WINNT_WINXP
#define WINVER _WIN32_WINNT_WINXP
#define NTDDI_VERSION NTDDI_WINXPSP2
#define _WIN32_IE _WIN32_IE_IE60

#include <sdkddkver.h>

#endif // _TARGETVER_H
